@@ -1,13 +1,13 @@
 /*
  * Main include file for Mathomatic.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #define	true	1
 #define	false	0
 
-#if	LIBRARY		/* define this when using Mathomatic as a symbolic math library */
+#if	LIBRARY
 #define	SILENT	1
 #endif
 
@@ -23,18 +23,15 @@
 #define	isfinite(d)	finite(d)
 #endif
 
-#undef	PI
-#define	PI	3.141592653589793238
-#define	E	2.718281828459045235
-
 #define	ARR_CNT(a)	(sizeof(a)/sizeof(a[0]))	/* returns the number of elements in an array */
 
-#define	MAX_K_INTEGER	1.0e14				/* 14 digits for doubles */
+#define	MAX_K_INTEGER	1.0e14				/* maximum representable integer, 14 digits for doubles */
+#define	MAX_PRECISION	16				/* maximum useful display precision (number of digits) */
 
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
 
-#if	I18N	/* internationalization */
+#if	I18N	/* internationalization, translators needed! */
 #define _(str)		gettext(str)
 #else
 #define _(str)		str
@@ -65,15 +62,16 @@
 /*
  * The following defines the default maximum mathematical expression size.
  * This should be made much smaller for Palmtops or embedded systems.
+ * Do not set to less than 100.
  */
 #define	DEFAULT_N_TOKENS	30000
 
-#define	DIVISOR_SIZE	(DEFAULT_N_TOKENS / 2)	/* A nice maximum divisor size. */
+#define	DIVISOR_SIZE	(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
 
-#define	MAX_VAR_NAMES	8000			/* Maximum number of long variable names. Keep this under (VAR_MASK - VAR_OFFSET). */
-#define	MAX_VAR_LEN	1000			/* Maximum size of long variable names */
+#define	MAX_VAR_NAMES	8000			/* maximum number of long variable names, keep this under (VAR_MASK - VAR_OFFSET) */
+#define	MAX_VAR_LEN	1000			/* maximum size of long variable names */
 
-#define	MAX_VARS	500			/* Maximum number of unique variables handled in each equation */
+#define	MAX_VARS	500			/* maximum number of unique variables handled in each equation */
 
 #define	VAR_OFFSET	'A'			/* makes space for predefined variables */
 #define	VAR_MASK	0x3fffL			/* mask for bits containing the variable name */
@@ -102,7 +100,7 @@ typedef union {
 #define	V_ANSWER	6L	/* the "answer" variable */
 #define V_TEMP		7L	/* the "temp" variable */
 #define	V_INTEGER	8L	/* the "integer" variable */
-	int	operatr;	/* internal storage for operators */
+	int	operatr;	/* internal storage for mathematical operators */
 /* valid operators follow */
 #define	PLUS		1	/* a + b */
 #define	MINUS		2	/* a - b */
