@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator commands.
+ * Mathomatic commands that don't belong anywhere else.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -9,17 +9,18 @@
 #define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in optimize command. */
 
 static int	sum_product();
-static int	find_more();
-static int	opt_es();
 static int	complex_func();
 static int	elim_sub();
+#if	!LIBRARY
 static int	edit_sub();
+static int	find_more();
+static int	opt_es();
 
 /* Global variables for the optimize command. */
 static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
+#endif
 
-#if	!LIBRARY
 /*
  * The version command.
  */
@@ -31,6 +32,9 @@ char	*cp;	/* the command line argument */
 		/* if anything on the command line, fail */
 		return false;
 	}
+#if	LIBRARY
+	result_str = strdup(VERSION);
+#endif
 	return version_report();
 }
 
@@ -53,12 +57,12 @@ version_report()
 #if	READLINE
 	fprintf(gfp, "READLINE ");
 #endif
-#if	BASICS
-	fprintf(gfp, "BASICS ");
-#endif
 #if	SILENT
 	fprintf(gfp, "SILENT ");
 #endif
+#if	LIBRARY
+	fprintf(gfp, "LIBRARY ");
+#endif
 #if	SECURE
 	fprintf(gfp, "SECURE ");
 #endif
@@ -71,7 +75,41 @@ version_report()
 	fprintf(gfp, "\n");
 	return true;
 }
-#endif
+
+/*
+ * The solve command.
+ */
+int
+solve_cmd(cp)
+char	*cp;
+{
+	int	i;
+	char	buf[MAX_CMD_LEN];
+
+	if (current_not_defined()) {
+		return false;
+	}
+	if (*cp == '\0') {
+		my_strlcpy(prompt_str, _("Enter variable: "), sizeof(prompt_str));
+		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
+			return false;
+		}
+		if (!case_sensitive_flag) {
+			str_tolower(cp);
+		}
+	}
+	i = next_espace();
+	if ((cp = parse_equation(i, cp)) != NULL) {
+		if (n_lhs[i] && n_rhs[i]) {
+			error(_("Invalid argument."));
+		} else if (!extra_garbage(cp) && solve(i, cur_equation)) {
+			return return_result(cur_equation);
+		}
+	}
+	n_lhs[i] = 0;
+	n_rhs[i] = 0;
+	return false;
+}
 
 /*
  * The sum command.
@@ -101,7 +139,7 @@ sum_product(cp, product_flag)
 char	*cp;		/* the command line */
 int	product_flag;	/* true for product, otherwise sum */
 {
-	int		i, j;
+	int		i;
 	long		v = 0;
 	double		start, end, step = 1.0;
 	int		result_en;
@@ -123,31 +161,21 @@ int	product_flag;	/* true for product, otherwise sum */
 		source = lhs[cur_equation];
 		dest = lhs[result_en];
 	}
-	if (*cp == '\0') {
-		for (j = 0; j < ns; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
-		if (v == 0) {
-			if (!prompt_var(&v))
-				return false;
-		}
-	} else {
+	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
 		}
 	}
+	if (no_vars(source, ns, &v)) {
+		error(_("Current expression contains no variables."));
+		return false;
+	}
+	if (v == 0) {
+		if (!prompt_var(&v)) {
+			return false;
+		}
+	}
 	if (!found_var(source, ns, v)) {
 		error(_("Variable not found."));
 		return false;
@@ -166,6 +194,9 @@ int	product_flag;	/* true for product, otherwise sum */
 		return false;
 	}
 	cp = skip_space(cp);
+	if (strcmp_tospace(cp, "to") == 0) {
+		cp = skip_param(cp);
+	}
 	if (*cp) {
 		cp1 = cp;
 	} else {
@@ -191,7 +222,7 @@ int	product_flag;	/* true for product, otherwise sum */
 		return false;
 	count_down = (end < start);
 	if (fmod(fabs(start - end) / step, 1.0)) {
-		printf(_("Warning: end value not reached.\n"));
+		error(_("Warning: end value not reached."));
 	}
 	if (product_flag) {
 		dest[0] = one_token;
@@ -236,30 +267,26 @@ int	product_flag;	/* true for product, otherwise sum */
 	} else {
 		n_lhs[result_en] = n;
 	}
-	return_result(result_en);
-	return true;
+	return return_result(result_en);
 }
 
-#if	!BASICS && !LIBRARY
+#if	!LIBRARY
 /*
  * This function is for the "optimize" command.
- * It finds and substitutes all occurrences of "en"
- * in "equation".
+ * It finds and substitutes all occurrences of the RHS of "en" in "equation".
  * It should be called repeatedly until it returns false.
  */
 static int
 find_more(equation, np, en)
-token_type	*equation;
-int		*np;		/* length of "equation" */
-int		en;		/* equation number */
+token_type	*equation;	/* expression pointer */
+int		*np;		/* pointer to length of expression */
+int		en;		/* equation space number */
 {
 	int	i, j, k;
 	int	level;
 	int	diff_sign;
-	int	found_se;
-	long	v;
+	int	found_se = true;
 
-	found_se = true;
 	for (level = 1; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
 			for (j = i; j < *np && equation[j].level > level; j += 2)
@@ -270,7 +297,6 @@ int		en;		/* equation number */
 			found_se = true;
 			k = i - 1;
 			if ((j - k) >= OPT_MIN_SIZE && se_compare(&equation[k], j - k, rhs[en], n_rhs[en], &diff_sign)) {
-				v = lhs[en][0].token.variable;
 				if (diff_sign) {
 					blt(&equation[i+2], &equation[j], (*np - j) * sizeof(token_type));
 					*np -= (j - (i + 2));
@@ -289,7 +315,7 @@ int		en;		/* equation number */
 				}
 				equation[k].level = level;
 				equation[k].kind = VARIABLE;
-				equation[k].token.variable = v;
+				equation[k].token.variable = lhs[en][0].token.variable;
 				return true;
 			}
 		}
@@ -299,8 +325,7 @@ int		en;		/* equation number */
 
 /*
  * This function is for the "optimize" command.
- * It finds and replaces all repeated expressions in
- * "equation" with temporary variables.
+ * It finds and replaces all repeated expressions in "equation" with temporary variables.
  * It also creates a new equation for each temporary variable.
  * It should be called repeatedly until it returns false.
  */
@@ -309,8 +334,7 @@ opt_es(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	i, j, k;
-	int	i1, i2, jj1, k1;
+	int	i, j, k, i1, i2, jj1, k1;
 	int	level, level1;
 	int	diff_sign;
 	int	found_se, found_se1;
@@ -403,9 +427,8 @@ int
 optimize_cmd(cp)
 char	*cp;
 {
-	int	i, j, k;
+	int	i, j, k, i1;
 	int	start, stop;
-	int	i1;
 	int	rv = false;
 	int	flag;
 
@@ -514,6 +537,11 @@ char	*cp;
  */
 output_options()
 {
+	if (!autosolve) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "autosolve\n");
+
 #if	!SILENT
 	fprintf(gfp, "debug_level = %d\n", debug_level);
 #endif
@@ -533,7 +561,7 @@ output_options()
 	}
 	fprintf(gfp, "bold_colors\n");
 
-	if (!groupall) {
+	if (!display2d) {
 		fprintf(gfp, "no ");
 	}
 	fprintf(gfp, "display2d\n");
@@ -583,7 +611,9 @@ int
 set_options(cp)
 char	*cp;
 {
+	int	i;
 	int	negate;
+	char	*cp1;
 
 next_option:
 	cp = skip_space(cp);
@@ -610,11 +640,13 @@ next_option:
 		if (negate) {
 			debug_level = 0;
 		} else {
-			if (*cp == '\0') {
-				error(_("Please specify a debug level number."));
+			i = decstrtol(cp, &cp1);
+			if (cp == cp1) {
+				error(_("Please specify the debug level number."));
 				return false;
 			}
-			debug_level = decstrtol(cp, &cp);
+			cp = cp1;
+			debug_level = i;
 		}
 		goto next_option;
 	}
@@ -624,14 +656,20 @@ next_option:
 		if (negate) {
 			screen_columns = 0;
 		} else {
-			if (*cp == '\0') {
+			if ((i = decstrtol(cp, &cp1)) < 0 || cp == cp1) {
 				error(_("Please specify how wide the screen is in columns."));
 				return false;
 			}
-			screen_columns = decstrtol(cp, &cp);
+			cp = cp1;
+			screen_columns = i;
 		}
 		goto next_option;
 	}
+	if (strncasecmp(cp, "autosolve", 9) == 0) {
+		cp = skip_param(cp);
+		autosolve = !negate;
+		goto next_option;
+	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
 		case_sensitive_flag = !negate;
@@ -639,7 +677,7 @@ next_option:
 	}
 	if (strncasecmp(cp, "display2d", 9) == 0) {
 		cp = skip_param(cp);
-		groupall = !negate;
+		display2d = !negate;
 		goto next_option;
 	}
 	if (strncasecmp(cp, "preserve", 8) == 0) {
@@ -824,8 +862,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	}
 	*np = n1;
 	cur_equation = j;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -866,6 +903,7 @@ char	*cp;
 	for (count = 0.0;; count++) {
 		fprintf(gfp, _("Running total = "));
 		list_proc(trhs, n_trhs, false);
+		fprintf(gfp, "\n");
 		if (count > 0.0) {
 			/* calculate and display the average */
 			blt(tlhs, trhs, n_trhs * sizeof(token_type));
@@ -885,10 +923,10 @@ char	*cp;
 			tlhs[n_tlhs].token.constant = count;
 			n_tlhs++;
 			calc_simp(tlhs, &n_tlhs);
-			fprintf(gfp, _(", average = "));
+			fprintf(gfp, _("Average = "));
 			list_proc(tlhs, n_tlhs, false);
+			fprintf(gfp, "\n");
 		}
-		fprintf(gfp, "\n");
 		my_strlcpy(prompt_str, _("Enter value: "), sizeof(prompt_str));
 		if (!get_expr(tlhs, &n_tlhs)) {
 			break;
@@ -1075,7 +1113,7 @@ char	*cp;
 			fprintf(gfp, " = ");
 		}
 		list_proc(tlhs, n_tlhs, false);
-		fprintf(gfp, "\n");
+		fprintf(gfp, "\n\n");
 	}
 	return true;
 }
@@ -1335,12 +1373,8 @@ char	*cp;
 	}
 	if (parse_complex(rhs[i], nr, &c1) && parse_complex(lhs[i], nl, &c2)) {
 		c3 = complex_div(c1, c2);
-		fprintf(gfp, _("Result of complex division: "));
-		if (c3.im == 0.0) {
-			fprintf(gfp, "%.12g\n\n", c3.re);
-		} else {
-			fprintf(gfp, "%.12g%+.12g*i#\n\n", c3.re, c3.im);
-		}
+		fprintf(gfp, _("Result of complex number division: "));
+		fprintf(gfp, "%.*g%+.*g*i#\n\n", precision, c3.re, precision, c3.im);
 		return true;
 	}
 	v_tmp = v;
@@ -1387,56 +1421,60 @@ char	*cp;
 	char	used[N_EQUATIONS];
 	int	vc = 0;
 
+	if (*cp == '\0') {
+		error(_("Please specify the variables to eliminate or \"all\" for all variables."));
+		return false;
+	}
 	for (i = 0; i < ARR_CNT(used); i++)
 		used[i] = false;
 	if (current_not_defined()) {
 		return false;
 	}
-	if (is_all(cp)) {
-		cp = skip_param(cp);
-		if (extra_garbage(cp))
-			return false;
-		last_v = 0;
-		for (;;) {
-			v1 = -1;
-			for (i = 0; i < n_lhs[cur_equation]; i += 2) {
-				if (lhs[cur_equation][i].kind == VARIABLE
-				    && lhs[cur_equation][i].token.variable > last_v) {
-					if (v1 == -1 || lhs[cur_equation][i].token.variable < v1) {
-						v1 = lhs[cur_equation][i].token.variable;
-					}
-				}
-			}
-			for (i = 0; i < n_rhs[cur_equation]; i += 2) {
-				if (rhs[cur_equation][i].kind == VARIABLE
-				    && rhs[cur_equation][i].token.variable > last_v) {
-					if (v1 == -1 || rhs[cur_equation][i].token.variable < v1) {
-						v1 = rhs[cur_equation][i].token.variable;
-					}
-				}
-			}
-			if (v1 == -1)
-				break;
-			last_v = v1;
-			if ((v1 & VAR_MASK) > SIGN) {
-				if (vc >= ARR_CNT(va)) {
-					break;
-				}
-				va[vc++] = v1;
-			}
-		}
-	}
 next_var:
-	if (*cp) {
+	if (vc) {
+		v = va[--vc];
+	} else if (*cp) {
+		if (is_all(cp)) {
+			cp = skip_param(cp);
+			vc = 0;
+			last_v = 0;
+			for (;;) {
+				v1 = -1;
+				for (i = 0; i < n_lhs[cur_equation]; i += 2) {
+					if (lhs[cur_equation][i].kind == VARIABLE
+					    && lhs[cur_equation][i].token.variable > last_v) {
+						if (v1 == -1 || lhs[cur_equation][i].token.variable < v1) {
+							v1 = lhs[cur_equation][i].token.variable;
+						}
+					}
+				}
+				for (i = 0; i < n_rhs[cur_equation]; i += 2) {
+					if (rhs[cur_equation][i].kind == VARIABLE
+					    && rhs[cur_equation][i].token.variable > last_v) {
+						if (v1 == -1 || rhs[cur_equation][i].token.variable < v1) {
+							v1 = rhs[cur_equation][i].token.variable;
+						}
+					}
+				}
+				if (v1 == -1)
+					break;
+				last_v = v1;
+				if ((v1 & VAR_MASK) > SIGN) {
+					if (vc >= ARR_CNT(va)) {
+						break;
+					}
+					va[vc++] = v1;
+				}
+			}
+			goto next_var;
+		}
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
 		}
-	} else if (vc) {
-		v = va[--vc];
 	} else {
 		if (did_something) {
-			return_result(cur_equation);
+			did_something = return_result(cur_equation);
 		} else {
 			error(_("No substitutions made."));
 		}
@@ -1482,7 +1520,6 @@ next_var:
 		}
 	}
 	did_something = true;
-	simp_sub(cur_equation);
 	used[i] = true;
 	goto next_var;
 }
@@ -1497,7 +1534,7 @@ long	v;
 #if	!SILENT
 	if (debug_level >= 0) {
 		list_var(v, 0);
-		printf(_("Solving equation #%d for (%s)...\n"), i + 1, var_str);
+		printf(_("Solving equation #%d for (%s) and substituting into the current equation...\n"), i + 1, var_str);
 	}
 #endif
 	want.level = 1;
@@ -1509,6 +1546,7 @@ long	v;
 	}
 	subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], rhs[i], n_rhs[i], v);
 	subst_var_with_exp(lhs[cur_equation], &n_lhs[cur_equation], rhs[i], n_rhs[i], v);
+	simp_sub(cur_equation);
 	return true;
 }
 
@@ -1524,7 +1562,8 @@ char	*cp;
 	int	i, j;
 	int	factor_flag;
 
-	if (factor_flag = (strcmp_tospace(cp, "factor") == 0)) {
+	factor_flag = (strcmp_tospace(cp, "factor") == 0);
+	if (factor_flag) {
 		cp = skip_param(cp);
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
@@ -1534,13 +1573,12 @@ char	*cp;
 		if (n_lhs[i] > 0) {
 			group_sub(i);
 			if (factor_flag || factor_int_flag) {
-				factor_int(lhs[i], &n_lhs[i]);
-				factor_int(rhs[i], &n_rhs[i]);
+				factor_int_sub(i);
 			}
 #if	LIBRARY
 			if (gfp == stdout) {
 				result_str = list_equation(i, false);
-				return true;
+				return(result_str != NULL);
 			}
 #endif
 			flist_sub(i);
@@ -1562,7 +1600,8 @@ char	*cp;
 	if (strcmp_tospace(cp, "export") == 0) {
 		export_flag = 2;
 		cp = skip_param(cp);
-	} else if (strcmp_tospace(cp, "maxima") == 0) {
+	}
+	if (strcmp_tospace(cp, "maxima") == 0) {
 		export_flag = 1;
 		cp = skip_param(cp);
 	}
@@ -1570,10 +1609,13 @@ char	*cp;
 		return false;
 	}
 	for (; i <= j; i++) {
+		if (factor_int_flag) {
+			factor_int_sub(i);
+		}
 #if	LIBRARY
 		if (gfp == stdout) {
 			result_str = list_equation(i, export_flag);
-			return true;
+			return(result_str != NULL);
 		}
 #endif
 		list1_sub(i, export_flag);
@@ -1581,7 +1623,7 @@ char	*cp;
 	return true;
 }
 
-#if	!BASICS && !LIBRARY
+#if	!LIBRARY
 /*
  * The code command.
  */
@@ -1590,17 +1632,17 @@ code_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	java_flag = 0;
-	int	int_flag = false;
+	int	language = 0;
+	int	int_flag = false, displayed = false;
 
-	if (strcmp_tospace(cp, "c") == 0) {
+	if (strcmp_tospace(cp, "c") == 0 || strcmp_tospace(cp, "c++") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "java") == 0) {
 		cp = skip_param(cp);
-		java_flag = 1;
+		language = 1;
 	} else if (strcmp_tospace(cp, "python") == 0) {
 		cp = skip_param(cp);
-		java_flag = 2;
+		language = 2;
 	} else if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
 		cp = skip_param(cp);
 		int_flag = true;
@@ -1611,30 +1653,27 @@ char	*cp;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
-#if	!SILENT
-				printf(_("#%d is not a solved equation.\n"), i + 1);
-#endif
+				error(_("Not a solved equation."));
 				continue;
 			}
-			in_calc_cmd = true;
+			approximate_roots = true;
+			simp_i(rhs[i], &n_rhs[i]);
+			approximate_roots = false;
 			if (int_flag) {
-				elim_loop(rhs[i], &n_rhs[i]);
 				uf_repeat_always(rhs[i], &n_rhs[i]);
 			}
 			group_sub(i);
-			in_calc_cmd = false;
 			if (int_flag) {
 				if (!int_expr(rhs[i], n_rhs[i])) {
-#if	!SILENT
-					printf(_("#%d is not an integer equation.\n"), i + 1);
-#endif
+					error(_("Not an integer equation."));
 					continue;
 				}
 			}
-			list_c_equation(i, java_flag, int_flag);
+			list_c_equation(i, language, int_flag);
+			displayed = true;
 		}
 	}
-	return true;
+	return displayed;
 }
 #endif
 
@@ -1665,9 +1704,10 @@ char	*cp;
 			return false;
 		subst_constants(lhs[i], &n_lhs[i]);
 		subst_constants(rhs[i], &n_rhs[i]);
+		approximate_roots = true;
 		simp_sub(i);
-		return_result(i);
-		return true;
+		approximate_roots = false;
+		return return_result(i);
 	}
 	for (vc = 0; *cp; vc++) {
 		if (strcmp_tospace(cp, "with") == 0) {
@@ -1731,8 +1771,7 @@ char	*cp;
 				if (!case_sensitive_flag) {
 					str_tolower(cp1);
 				}
-				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL
-				    || n <= 0) {
+				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL || n <= 0) {
 					return false;
 				}
 				if (extra_garbage(cp1))
@@ -1765,8 +1804,7 @@ do_this:
 	n_rhs[i] = n_trhs;
 	blt(rhs[i], trhs, n_trhs * sizeof(token_type));
 	simp_sub(i);
-	return_result(i);
-	return true;
+	return return_result(i);
 }
 
 /*
@@ -1870,7 +1908,8 @@ char	*cp;
 	int	i, j;
 	int	fully_flag;
 
-	if (fully_flag = (strncasecmp(cp, "fully", 4) == 0)) {
+	fully_flag = (strncasecmp(cp, "fully", 4) == 0);
+	if (fully_flag) {
 		cp = skip_param(cp);
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
@@ -1888,6 +1927,28 @@ char	*cp;
 	return true;
 }
 
+/*
+ * The fraction command.
+ */
+int
+fraction_cmd(cp)
+char	*cp;
+{
+	int	i, j;
+
+	if (!get_range_eol(&cp, &i, &j)) {
+		return false;
+	}
+	for (; i <= j; i++) {
+		if (n_lhs[i]) {
+			frac_side(lhs[i], &n_lhs[i]);
+			frac_side(rhs[i], &n_rhs[i]);
+			return_result(i);
+		}
+	}
+	return true;
+}
+
 #if	!LIBRARY
 /*
  * The quit command.
@@ -1938,7 +1999,7 @@ char	*cp;
 		printf(_("Read operation aborted.\n"));
 		goto end_read;
 	}
-	while (cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) {
+	while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
 		default_color();
 		input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
 		printf("%s", cp);
@@ -2051,6 +2112,7 @@ char	*cp;
 		error(_("No file specified."));
 		return false;
 	}
+#if	!SILENT
 	if (access(cp, 0) == 0) {
 		snprintf(prompt_str, sizeof(prompt_str), _("\"%s\" exists.  Overwrite (Y/N)? "), cp);
 		if (!get_yes_no()) {
@@ -2058,6 +2120,7 @@ char	*cp;
 			return false;
 		}
 	}
+#endif
 	fp = fopen(cp, "w");
 	if (fp == NULL) {
 		error(_("Can't create file."));
@@ -2071,7 +2134,9 @@ char	*cp;
 	if (fclose(fp))
 		rv = false;
 	if (rv) {
+#if	!SILENT
 		printf(_("All equations saved in file: \"%s\".\n"), cp);
+#endif
 	} else {
 		error(_("Error encountered while saving equations."));
 	}
