@@ -2,7 +2,7 @@
  * An independent, floating point complex number library for C.
  * Include "complex.h" if you use this.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "complex.h"
@@ -14,13 +14,13 @@
 #define epsilon	0.00000000000005	/* a good value for doubles */
 
 /*
- * Zero out relatively very small real or imaginary parts.
+ * Zero out relatively very small real or imaginary parts of a complex number.
  *
  * Return true if something was zeroed out.
  */
 int
 complex_fixup(ap)
-complexs	*ap;
+complexs	*ap;	/* complex number pointer */
 {
 	if (fabs(ap->re * epsilon) > fabs(ap->im)) {
 		ap->im = 0.0;
@@ -76,7 +76,7 @@ complexs	b;
 }
 
 /*
- * Divide complex "a" by complex "b" and return result.
+ * Divide complex number "a" by complex number "b" and return result.
  */
 complexs
 complex_div(a, b)
