@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator differentiation routines and commands.
+ * Mathomatic differentiation routines and commands.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -74,7 +74,7 @@ long		v;
 		return true;
 	}
 	op = 0;
-	for (endloc = loc + 1; endloc < *np && equation[endloc].level >= level; endloc += 2) {
+	for (oploc = endloc = loc + 1; endloc < *np && equation[endloc].level >= level; endloc += 2) {
 		if (equation[endloc].level == level) {
 			switch (op) {
 			case 0:
@@ -289,7 +289,7 @@ derivative_cmd(cp)
 char	*cp;
 {
 	long		v = 0;
-	int		i, j;
+	int		i;
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
@@ -314,7 +314,7 @@ char	*cp;
 			cp = skip_param(cp);
 			v = MATCH_ANY;
 		} else {
-			if (!(isascii(*cp) && isdigit(*cp))) {
+			if (!isdigit(*cp)) {
 				cp = parse_var2(&v, cp);
 				if (cp == NULL) {
 					return false;
@@ -329,21 +329,9 @@ char	*cp;
 			return false;
 		}
 	}
-	if (v == 0) {
-		for (j = 0; j < *nps; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
+	if (no_vars(source, *nps, &v)) {
+		error(_("Current expression contains no variables, result would be zero."));
+		return false;
 	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
@@ -351,7 +339,7 @@ char	*cp;
 		}
 	}
 	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
-		error(_("Variable not found.  Result would be zero."));
+		error(_("Variable not found, result would be zero."));
 		return false;
 	}
 #if	!SILENT
@@ -378,8 +366,7 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -389,7 +376,7 @@ int
 extrema_cmd(cp)
 char	*cp;
 {
-	int		i, j;
+	int		i;
 	long		v = 0;
 	token_type	want;
 	token_type	*source;
@@ -418,21 +405,9 @@ char	*cp;
 			return false;
 		}
 	}
-	if (v == 0) {
-		for (j = 0; j < n; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
+	if (no_vars(source, n, &v)) {
+		error(_("Current expression contains no variables, there are no extrema."));
+		return false;
 	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
@@ -445,19 +420,22 @@ char	*cp;
 	}
 	i = next_espace();
 	blt(rhs[i], source, n * sizeof(token_type));
-	n_rhs[i] = n;
-	if (!differentiate(rhs[i], &n_rhs[i], v)) {
+/* take the derivative with respect to the specified variable and simplify: */
+	if (!differentiate(rhs[i], &n, v)) {
 		error(_("Differentiation failed."));
 		return false;
 	}
-	simpa_side(rhs[i], &n_rhs[i], true);
-	if (!found_var(rhs[i], n_rhs[i], v)) {
+	simpa_side(rhs[i], &n, true);
+	if (!found_var(rhs[i], n, v)) {
 		error(_("There are no extrema for this equation."));
 		return false;
 	}
+	n_rhs[i] = n;
+/* set equal to zero: */
 	n_lhs[i] = 1;
 	lhs[i][0] = zero_token;
 	cur_equation = i;
+/* lastly, solve for the specified variable and simplify: */
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
@@ -466,8 +444,7 @@ char	*cp;
 		return false;
 	}
 	simpa_side(rhs[i], &n_rhs[i], false);
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -513,7 +490,7 @@ char	*cp;
 		dest = lhs[i];
 		np = &n_lhs[i];
 	}
-	if (*cp && !(isascii(*cp) && isdigit(*cp))) {
+	if (*cp && !isdigit(*cp)) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
@@ -527,25 +504,14 @@ char	*cp;
 		}
 		cp = cp1;
 	}
-	if (v == 0) {
-		for (j = 0; j < *nps; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if ((source[j].token.variable & VAR_MASK) <= SIGN)
-					continue;
-				if (v) {
-					if (v != source[j].token.variable) {
-						v = 0;
-						break;
-					}
-				} else {
-					v = source[j].token.variable;
-				}
-			}
-		}
+	if (no_vars(source, *nps, &v)) {
+		error(_("Current expression contains no variables."));
+		return false;
 	}
 	if (v == 0) {
-		if (!prompt_var(&v))
+		if (!prompt_var(&v)) {
 			return false;
+		}
 	}
 	if (!found_var(source, *nps, v)) {
 		error(_("Variable not found."));
@@ -572,10 +538,13 @@ char	*cp;
 		list_var(v, 0);
 		printf(_("Taylor series expansion about (%s) = point.\n"), var_str);
 #endif
-		my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
-		if (!get_expr(lhs[our], &our_nlhs)) {
-			error(_("Enter 0 if you don't know."));
-			return false;
+		for (;;) {
+			my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
+			if (!get_expr(lhs[our], &our_nlhs)) {
+				error(_("Enter \"0\" if you don't know."));
+				continue;
+			}
+			break;
 		}
 	}
 	if (order < 0) {
@@ -657,7 +626,7 @@ loop_again:
 	for (; i1 < n1; i1++)
 		dest[i1].level++;
 	simp_loop(dest, &n1);
-	side_debug(1, dest, n1);
+/*	side_debug(1, dest, n1); */
 	if (n < order) {
 		if (n > 0) {
 			if (!differentiate(rhs[our], &our_nrhs, v)) {
@@ -687,8 +656,7 @@ loop_again:
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -699,7 +667,7 @@ limit_cmd(cp)
 char	*cp;
 {
 	int		i, j, k;
-	long		v;
+	long		v = 0;
 	token_type	solved_v, want;
 	char		*cp_start;
 	int		nl;
@@ -711,6 +679,7 @@ char	*cp;
 		return false;
 	}
 	if (n_rhs[cur_equation] == 0) {
+/* make expression into equation: */
 		blt(rhs[cur_equation], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_rhs[cur_equation] = n_lhs[cur_equation];
 		n_lhs[cur_equation] = 1;
@@ -729,7 +698,12 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-	} else {
+	}
+	if (no_vars(rhs[cur_equation], n_rhs[cur_equation], &v)) {
+		error(_("Current expression contains no variables."));
+		return false;
+	}
+	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
@@ -747,7 +721,8 @@ char	*cp;
 			return false;
 		}
 	} else {
-		my_strlcpy(prompt_str, _("goes to: "), sizeof(prompt_str));
+		list_var(v, 0);
+		snprintf(prompt_str, sizeof(prompt_str), _("as (%s) goes to: "), var_str);
 		if (!get_expr(lhs[i], &nl)) {
 			return false;
 		}
@@ -796,12 +771,14 @@ char	*cp;
 		}
 	}
 	if (!found_den || (infinity_flag && !complex_den)) {
+/* in this case, just substitute the limit variable with the limit expression and simplify: */
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], lhs[i], nl, v);
 		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
-		return_result(cur_equation);
-		return true;
+		return return_result(cur_equation);
 	}
 	if (infinity_flag) {
+/* To take the limit to infinity, */
+/* replace infinity with zero and replace the limit variable with its reciprocal: */
 		nl = 1;
 		lhs[i][0] = zero_token;
 		tlhs[0] = one_token;
@@ -814,6 +791,7 @@ char	*cp;
 		n_tlhs = 3;
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tlhs, n_tlhs, v);
 	}
+/* General limit taking, solve for the limit variable: */
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
@@ -821,18 +799,21 @@ char	*cp;
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
+/* replace the limit variable (LHS) with the limit expression: */
 	blt(lhs[cur_equation], lhs[i], nl * sizeof(token_type));
 	n_lhs[cur_equation] = nl;
+/* symbolically simplify the RHS: */
 	symb_flag = true;
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	symb_flag = false;
+/* solve back for the original variable: */
 	if (solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
+/* symbolically simplify before returning the result: */
 	symb_flag = true;
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	symb_flag = false;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
