@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator externals.
+ * Mathomatic global variable definitions.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 extern int		n_tokens;
@@ -27,22 +27,24 @@ extern token_type	one_token;
 
 extern char		*var_names[MAX_VAR_NAMES];
 
+extern int		precision;
 extern int		case_sensitive_flag;
 extern int		factor_int_flag;
-extern int		groupall;
-extern int		in_calc_cmd;
+extern int		display2d;
+extern int		approximate_roots;
 extern int		preserve_roots;
 extern int		true_modulus;
 extern int		screen_columns;
 extern int		screen_rows;
 extern int		finance_option;
-extern int		starstar;
+extern int		autosolve;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
 extern int		bold_colors;
 extern int		cur_color;
 extern int		html_flag;
+extern int		readline_enabled;
 extern int		partial_flag;
 extern int		symb_flag;
 extern int		high_prec;
