@@ -5,7 +5,7 @@
 def fact(x):
 	"Return x! (x factorial)."
 	if (x < 0 or (x % 1.0) != 0.0):
-		raise FloatingPointError, "Factorial argument must be a positive integer."
+		raise ValueError, "Factorial argument must be a positive integer."
 		return
 	if (x == 0):
 		return x + 1
