@@ -15,7 +15,7 @@ int	arg;
 	for (i = 1; i > 0 && arg > 1; arg--) {
 		i *= arg;
 	}
-	if (i >= 0) {
+	if (i > 0) {
 		return i;
 	} else {
 		return 0;
