@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator constant factorizing routines.
+ * Mathomatic constant factorizing routines.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -9,6 +9,7 @@
 static void	try_factor();
 static int	fc_recurse();
 
+/* The following data is used to factor integers: */
 static double nn, vv;
 static double sq[] = {		/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
@@ -21,7 +22,7 @@ static double sq[] = {		/* Additive array that skips over multiples of 2, 3, 5,
 
 /*
  * Factor the integer in "start".
- * Store unique factors in the "unique[]" array.
+ * Store the prime factors in the unique[] array.
  *
  * Return true if successful.
  */
@@ -30,12 +31,13 @@ factor_one(start)
 double	start;
 {
 	int	i;
-	double	ii;
+	double	d;
 
 	uno = 0;
 	nn = start;
-	if (nn == 0.0)
+	if (nn == 0.0) {
 		return false;
+	}
 	if (fabs(nn) >= MAX_K_INTEGER) {
 		/* too large to factor */
 		return false;
@@ -49,16 +51,20 @@ double	start;
 	try_factor(3.0);
 	try_factor(5.0);
 	try_factor(7.0);
-	ii = 1.0;
-	while (ii <= vv) {
+	d = 1.0;
+	while (d <= vv) {
 		for (i = 0; i < ARR_CNT(sq); i++) {
-			ii += sq[i];
-			try_factor(ii);
+			d += sq[i];
+			try_factor(d);
 		}
 	}
 	if (nn != 1.0) {
 		try_factor(nn);
 	}
+	if (start != multiply_out_unique()) {
+		error("Error factoring integers.");
+		return false;
+	}
 	return true;
 }
 
@@ -105,7 +111,7 @@ multiply_out_unique()
 }
 
 /*
- * Display the factors in the unique[] array.
+ * Display the prime factors in the unique[] array.
  */
 display_unique()
 {
@@ -196,6 +202,16 @@ int		*np;
 	return modified;
 }
 
+/*
+ * Factor integers in an equation space.
+ */
+factor_int_sub(n)
+int	n;	/* equation space number */
+{
+	factor_int(lhs[n], &n_lhs[n]);
+	factor_int(rhs[n], &n_rhs[n]);
+}
+
 /*
  * Factor constants in equation side.
  *
@@ -268,7 +284,7 @@ break_cont:
 				if (first) {
 					minimum = d;
 					first = false;
-				} else if (d < minimum) {
+				} else if (minimum > d) {
 					minimum = d;
 				}
 				break;
@@ -289,15 +305,14 @@ break_cont:
 				if (first) {
 					minimum = 1.0;
 					first = false;
-				} else if (1.0 < minimum)
+				} else if (minimum > 1.0) {
 					minimum = 1.0;
+				}
 				break;
 			}
 		} else {
 			op = 0;
-			for (j = i + 1;; j += 2) {
-				if (j >= *np || equation[j].level <= level)
-					break;
+			for (j = i + 1; j < *np && equation[j].level > level; j += 2) {
 				if (equation[j].level == level + 1) {
 					op = equation[j].token.operatr;
 				}
@@ -375,7 +390,7 @@ break_cont:
 				if (j >= *np || equation[j].level <= level)
 					break;
 			}
-			blt(&equation[j+2], &equation[j], (*np - j) * sizeof(*equation));
+			blt(&equation[j+2], &equation[j], (*np - j) * sizeof(token_type));
 			*np += 2;
 			equation[j].level = level + 1;
 			equation[j].kind = OPERATOR;
@@ -390,7 +405,7 @@ break_cont:
 	for (i = loc; i < *np && equation[i].level >= level; i++) {
 		equation[i].level++;
 	}
-	blt(&equation[i+2], &equation[i], (*np - i) * sizeof(*equation));
+	blt(&equation[i+2], &equation[i], (*np - i) * sizeof(token_type));
 	*np += 2;
 	equation[i].level = level;
 	equation[i].kind = OPERATOR;
