@@ -1,7 +1,7 @@
 /*
- * Algebraic manipulator globals.
+ * Mathomatic global variables.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  *
  * All global variables for Mathomatic are defined here and duplicated in "externs.h".
  */
@@ -30,16 +30,16 @@ token_type	zero_token;			/* the constant 0.0 as a token */
 token_type	one_token;			/* the constant 1.0 as a token */
 
 /* set options */
+int		precision = 14;			/* the display precision (number of digits) */
 int		case_sensitive_flag = true;	/* "set case_sensitive" flag */
 int		factor_int_flag;		/* factor integers when displaying expressions */
-int		groupall = true;		/* "set display2d" flag */
-int		preserve_roots = true;		/* true if we are to preserve 2^.5, etc. */
-int		in_calc_cmd;			/* true if in calculate command (force approximations) */
+int		display2d = true;		/* "set display2d" flag */
+int		preserve_roots = true;		/* set option to preserve 2^.5, etc. */
 int		true_modulus;			/* true for mathematically correct modulus */
 int		screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width */
 int		screen_rows = STANDARD_SCREEN_ROWS;		/* screen height */
 int		finance_option;			/* for displaying dollars and cents */
-int		starstar;			/* output "**" instead of "^" for the power operator */
+int		autosolve = true;		/* Allows solving by just typing the variable name at the prompt */
 #if	!SILENT
 int		debug_level;			/* current debug level */
 #endif
@@ -63,7 +63,7 @@ char		*dir_path;			/* directory path to the executable */
 char		*prog_name = "mathomatic";	/* name of this program */
 
 /* The following are for integer factoring (filled by factor_one()): */
-double		unique[64];		/* storage for the unique factors */
+double		unique[64];		/* storage for the unique prime factors */
 int		ucnt[64];		/* number of times the factor occurs */
 int		uno;			/* number of unique factors stored in unique[] */
 
@@ -74,13 +74,17 @@ jmp_buf		jmp_save;		/* for setjmp() when an error is encountered */
 int		test_mode;		/* test mode flag (-t) */
 int		quiet_mode;		/* quiet mode (don't display prompts) */
 int		html_flag;		/* true for HTML mode */
+int		readline_enabled = true;
 int		partial_flag;		/* true for partial unfactoring of algebraic fractions */
 int		symb_flag;		/* true for "simplify symbolic" */
 int		high_prec;		/* flag to output constants in higher precision (used when saving equations) */
 int		input_column;		/* current column number on the screen at the beginning of a parse */
 int		sign_flag;		/* true when all "sign" variables are to compare equal */
 int		domain_check;		/* flag to track domain errors in the pow() function */
+int		approximate_roots;	/* true if in calculate command (force approximations) */
 
 /* library variables go here */
+#if	SILENT
 char		*error_str;		/* last error when SILENT is defined */
 char		*result_str;		/* returned result when using as library */
+#endif
