@@ -3,7 +3,7 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -12,79 +12,81 @@
  * The following structure is used for each Mathomatic command.
  */
 typedef	struct {
-	char	*name;		/* command name to be typed by user */
-	int	(*func)();	/* function that handles this command */
-				/* function is passed a char pointer and returns true if successful */
-	char	*usage;		/* command syntax text */
-	char	*info;		/* one line description of command */
+	char	*name;			/* command name to be typed by user */
+	char	*secondary_name;	/* another name for this command */
+	int	(*func)();		/* function that handles this command */
+					/* function is passed a char pointer and returns true if successful */
+	char	*usage;			/* command syntax text */
+	char	*info;			/* one line description of command */
 } com_type;
 
 /*
  * The Mathomatic command table follows.  It should be in alphabetical order.
  */
 static com_type com_list[] = {
+/*	command name,	alternate name,		function,	usage,							information */
 #if	!LIBRARY
-{	"calculate",	calculate_cmd,	"[variable number-of-iterations]",			"Temporarily plug in values for variables and approximate." },
+{	"calculate",	NULL,			calculate_cmd,	"[variable number-of-iterations]",			"Temporarily plug in values for variables and approximate." },
 #endif
-{	"clear",	clear_cmd,	"[equation-numbers-or-ranges]",				"Delete expressions or equations so equation spaces can be reused." },
-#if	!BASICS && !LIBRARY
-{	"code",		code_cmd,	"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-range]",	"Output C, Java, or Python code for the specified equations." },
-#endif
-{	"compare",	compare_cmd,	"equation-number [\"with\" equation-number]",		"Compare two expressions or equations to see if they are the same." },
-{	"copy",		copy_cmd,	"[equation-number-range]",				"Duplicate the specified expressions or equations." },
-{	"derivative",	derivative_cmd,	"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+{	"clear",	NULL,			clear_cmd,	"[equation-numbers-or-ranges]",				"Delete expressions or equations so equation spaces can be reused." },
 #if	!LIBRARY
-{	"divide",	divide_cmd,	"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
+{	"code",		NULL,			code_cmd,	"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-range]",	"Output C, Java, or Python code for the specified equations." },
+#endif
+{	"compare",	NULL,			compare_cmd,	"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
+{	"copy",		NULL,			copy_cmd,	"[equation-number-range]",				"Duplicate the specified equation spaces." },
+{	"derivative",	"differentiate",	derivative_cmd,	"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+#if	!LIBRARY
+{	"divide",	NULL,			divide_cmd,	"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
 #if	(UNIX || CYGWIN) && !SECURE
-{	"edit",		edit_cmd,	"[file-name]",						"Edit all stored expressions and equations or an input file." },
+{	"edit",		NULL,			edit_cmd,	"[file-name]",						"Edit all equation spaces or an input file." },
 #endif
 #endif
-{	"eliminate",	eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
-{	"extrema",	extrema_cmd,	"[variable]",						"Find the minimums and maximums of the current expression." },
-{	"factor",	factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Trivially factor integers or stored expressions." },
-{	"flist",	flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
-{	"help",		help_cmd,	"[topic or command-name]",				"Short, built-in help." },
-{	"imaginary",	imaginary_cmd,	"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
-{	"integrate",	integrate_cmd,	"[\"definite\"] variable [order]",			"Symbolically integrate polynomials, order times." },
-{	"laplace",	laplace_cmd,	"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
-{	"limit",	limit_cmd,	"variable expression",					"Take the limit of the current expression as variable goes to expression." },
-{	"list",		list_cmd,	"[\"export\" or \"maxima\"] [equation-number-range]",	"Display stored expressions in single line format." },
-#if	!BASICS && !LIBRARY
-{	"nintegrate",	nintegrate_cmd,	"[\"trapezoidal\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
-{	"optimize",	optimize_cmd,	"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
+{	"eliminate",	NULL,			eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
+{	"extrema",	"extremum",		extrema_cmd,	"[variable]",						"Find the minimums and maximums of the current expression." },
+{	"factor",	NULL,			factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Trivially factor integers or stored expressions." },
+{	"flist",	"display",		flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
+{	"fraction",	NULL,			fraction_cmd,	"[equation-number-range]",				"Convert expressions with any algebraic fractions into a single fraction." },
+{	"help",		NULL,			help_cmd,	"[topic or command-name]",				"Short, built-in help." },
+{	"imaginary",	NULL,			imaginary_cmd,	"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
+{	"integrate",	"integral",		integrate_cmd,	"[\"definite\"] variable [order]",			"Symbolically integrate polynomials, order times." },
+{	"laplace",	NULL,			laplace_cmd,	"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
+{	"limit",	NULL,			limit_cmd,	"variable expression",					"Take the limit of the current expression as variable goes to expression." },
+{	"list",		NULL,			list_cmd,	"[\"export\" or \"maxima\"] [equation-number-range]",	"Display stored expressions in single line format." },
+#if	!LIBRARY
+{	"nintegrate",	NULL,			nintegrate_cmd,	"[\"trapezoidal\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
+{	"optimize",	NULL,			optimize_cmd,	"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
 #endif
 #if	!LIBRARY
-{	"pause",	pause_cmd,	"[text]",						"Wait for user to press the Enter key.  Optionally display a message." },
+{	"pause",	NULL,			pause_cmd,	"[text]",						"Wait for user to press the Enter key.  Optionally display a message." },
 #endif
-{	"product",	product_cmd,	"variable start end [step]",				"Compute the product as variable goes from start to end." },
+{	"product",	NULL,			product_cmd,	"variable start end [step]",				"Compute the product as variable goes from start to end." },
 #if	READLINE
-{	"push",		push_cmd,	"[equation-number-range]",				"Push stored expressions into the readline history." },
+{	"push",		NULL,			push_cmd,	"[equation-number-range]",				"Push stored expressions into the readline history." },
 #endif
 #if	!LIBRARY
-{	"quit",		quit_cmd,	"",							"Terminate this program." },
+{	"quit",		"exit",			quit_cmd,	"",							"Terminate this program without saving." },
 #endif
 #if	!SECURE && !LIBRARY
-{	"read",		read_cmd,	"file-name",						"Read in a text file as if it was typed in." },
+{	"read",		NULL,			read_cmd,	"file-name",						"Read in a text file as if it was typed in." },
 #endif
-{	"real",		real_cmd,	"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
-{	"replace",	replace_cmd,	"[\"constants\"] [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
+{	"real",		NULL,			real_cmd,	"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
+{	"replace",	NULL,			replace_cmd,	"[\"constants\"] or [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
 #if	!LIBRARY
-{	"roots",	roots_cmd,	"root real-part imaginary-part",			"Display all the roots of a complex number." },
+{	"roots",	NULL,			roots_cmd,	"root real-part imaginary-part",			"Display all the roots of a complex number." },
 #endif
 #if	!SECURE
-{	"save",		save_cmd,	"file-name",						"Save all stored expressions and equations in a text file." },
+{	"save",		NULL,			save_cmd,	"file-name",						"Save all stored expressions and equations in a text file." },
 #endif
-{	"set",		set_cmd,	"[[\"no\"] options]",					"Set or display various session options." },
-{	"simplify",	simplify_cmd,	"[\"symbolic\"] [\"quick\"] [equation-number-range]",	"Completely simplify stored expressions." },
-{	"sum",		sum_cmd,	"variable start end [step]",				"Compute the summation as variable goes from start to end." },
+{	"set",		NULL,			set_cmd,	"[[\"no\"] options]",					"Set or display various session options." },
+{	"simplify",	NULL,			simplify_cmd,	"[\"symbolic\"] [\"quick\"] [equation-number-range]",	"Completely simplify stored expressions." },
+{	"solve",	NULL,			solve_cmd,	"variable or \"0\"",					"Solve the current equation for a variable or for 0." },
+{	"sum",		NULL,			sum_cmd,	"variable start end [step]",				"Compute the summation as variable goes from start to end." },
 #if	!LIBRARY
-{	"tally",	tally_cmd,	"",							"Prompt for and add numerical entries.  Show running total and average." },
-#endif
-{	"taylor",	taylor_cmd,	"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
-{	"unfactor",	unfactor_cmd,	"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) stored expressions." },
-#if	!LIBRARY
-{	"version",	version_cmd,	"",							"Display version number and compile flags used." }
+{	"tally",	NULL,			tally_cmd,	"",							"Prompt for and add numerical entries, show running total and average." },
 #endif
+{	"taylor",	NULL,			taylor_cmd,	"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
+{	"unfactor",	"expand",		unfactor_cmd,	"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) stored expressions." },
+{	"version",	NULL,			version_cmd,	"",							"Display version number and compile flags used." }
 };
 
 /*
@@ -103,11 +105,13 @@ char	*cp;
 	if ((cp = parse_equation(n, cp)) != NULL) {
 		if (n_lhs[n] == 0 && n_rhs[n] == 0)
 			return true;
-		if (n_lhs[n]) {
-			cur_equation = n;
-			return_result(cur_equation);
-			return true;
+		if (n_lhs[n] == 0) {
+			/* RHS expression only, set equal to zero */
+			n_lhs[n] = 1;
+			lhs[n][0] = zero_token;
 		}
+		cur_equation = n;
+		return return_result(cur_equation);
 	}
 	n_lhs[n] = 0;
 	n_rhs[n] = 0;
@@ -127,23 +131,32 @@ process_parse(n, cp)
 int	n;
 char	*cp;
 {
+#if	!LIBRARY
 	int	i;
+#endif
 
 	if ((cp = parse_equation(n, cp)) != NULL) {
 		if (n_lhs[n] == 0 && n_rhs[n] == 0)
 			return true;
 		if (n_lhs[n] == 0 || n_rhs[n] == 0) {
-			if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
-			    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
-			    || n_rhs[n]) {
-				if (solve(n, cur_equation)) {
-					return_result(cur_equation);
-					return true;
+			if (autosolve) {
+				if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
+				    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
+				    || (n_rhs[n] == 1 && ((rhs[n][0].kind == CONSTANT && rhs[n][0].token.constant == 0.0)
+				    || (rhs[n][0].kind == VARIABLE && (rhs[n][0].token.variable & VAR_MASK) > SIGN)))) {
+					if (solve(n, cur_equation)) {
+						return return_result(cur_equation);
+					}
+					return false;
 				}
-				return false;
 			}
-			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT
-			    && fmod(lhs[n][0].token.constant, 1.0) == 0.0
+			if (n_rhs[n]) {
+				/* RHS expression only, set equal to zero */
+				n_lhs[n] = 1;
+				lhs[n][0] = zero_token;
+				goto return_ok;
+			}
+			if (n_lhs[n] == 1 && lhs[n][0].kind == CONSTANT && fmod(lhs[n][0].token.constant, 1.0) == 0.0
 			    && lhs[n][0].token.constant > 0.0 && lhs[n][0].token.constant <= n_equations) {
 				/* easy selecting of equation spaces */
 				n_lhs[n] = 0;
@@ -175,8 +188,7 @@ char	*cp;
 		}
 return_ok:
 		cur_equation = n;
-		return_result(cur_equation);
-		return true;
+		return return_result(cur_equation);
 	}
 	n_lhs[n] = 0;
 	n_rhs[n] = 0;
@@ -213,15 +225,10 @@ char	*cp;
 		cp++;
 		switch (*cp) {
 		case '+':
-			cp++;
+		case '-':
 			i = decstrtol(cp, &cp1);
 			i = cur_equation + i;
 			break;
-		case '-':
-			cp++;
-			i = decstrtol(cp, &cp1);
-			i = cur_equation - i;
-			break;
 		default:
 			i = decstrtol(cp, &cp1) - 1;
 			break;
@@ -242,7 +249,7 @@ char	*cp;
 			return parse(i, cp);
 		}
 		cur_equation = i;
-		return_result(cur_equation);
+		return return_result(cur_equation);
 	}
 #if	!SECURE
 	/* handle shell escape */
@@ -255,23 +262,24 @@ char	*cp;
 		if (cp1) {
 			return(!shell_out(cp1));
 		}
+		error(_("SHELL environment variable not set."));
 		return false;
 	}
 #endif
+	/* a quick way to get help */
+	if (*cp == '?') {
+		cp = skip_space(cp + 1);
+		input_column += (cp - cp_start);
+		return(help_cmd(cp));
+	}
 /* See if the string pointed to by "cp" is a command. */
 /* If so, execute it. */
 	cp1 = cp;
-	while (*cp1 && isascii(*cp1) && isalpha(*cp1))
+	while (*cp1 && isalpha(*cp1))
 		cp1++;
 	for (i = 0; i < ARR_CNT(com_list); i++) {
-		if (strlen(com_list[i].name) < 4) {
-			if ((cp1 - cp) != strlen(com_list[i].name))
-				continue;
-		} else {
-			if ((cp1 - cp) < 4)
-				continue;
-		}
-		if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0) {
+		if (((cp1 - cp) >= min(4, strlen(com_list[i].name)) && strncasecmp(cp, com_list[i].name, cp1 - cp) == 0)
+		    || (com_list[i].secondary_name && (cp1 - cp) >= min(4, strlen(com_list[i].secondary_name)) && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
 			cp1 = skip_space(cp1);
 			input_column += (cp1 - cp_start);
 			if (my_strlcpy(buf2, cp1, sizeof(buf2)) >= sizeof(buf2)) {
@@ -311,7 +319,7 @@ char	*cp;
 #endif
 			/* remove trailing spaces from the command line */
 		        i1 = strlen(buf2) - 1;
-		        while (i1 >= 0 && isascii(buf2[i1]) && isspace(buf2[i1])) {
+		        while (i1 >= 0 && isspace(buf2[i1])) {
         		        buf2[i1] = '\0';
                 		i1--;
 			}
@@ -355,19 +363,18 @@ char	*cp;
 #endif
 
 /*
- * Parse a variable name, with error messages and space character skipping.
+ * Parse a variable name with before and after space character skipping.
  *
  * Return new position in string or NULL if error.
  */
 char	*
 parse_var2(vp, cp)
-long	*vp;	/* pointer to result */
+long	*vp;	/* pointer to returned variable in Mathomatic internal format */
 char	*cp;	/* pointer to variable name string */
 {
 	cp = skip_space(cp);
 	cp = parse_var(vp, cp);
 	if (cp == NULL) {
-		error(_("Invalid variable."));
 		return NULL;
 	}
 	return skip_space(cp);
@@ -378,6 +385,7 @@ char	*cp;	/* pointer to variable name string */
 /*
  * The help command.
  */
+int
 help_cmd(cp)
 char	*cp;
 {
@@ -387,16 +395,15 @@ char	*cp;
 	int	row;
 
 	cp1 = cp;
-	while (*cp1 && !(isascii(*cp1) && isspace(*cp1)))
+	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	if (cp1 != cp) {
 		/* first, see if the argument matches any command names */
 		flag = false;
 		for (i = 0; i < ARR_CNT(com_list); i++) {
-			if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0) {
-				if (com_list[i].info) {
-					fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
-				}
+			if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0
+			    || (com_list[i].secondary_name && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
+				fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
 				fprintf(gfp, "Usage: %s %s\n\n", com_list[i].name, com_list[i].usage);
 				flag = true;
 			}
@@ -417,7 +424,7 @@ char	*cp;
 			P("Rectangle of length \"l\" and width \"w\":");
 			P("    Area = l*w                    Perimeter = 2*l + 2*w");
 			P("Trapezoid of height \"h\" and parallel sides \"a\" and \"b\":");
-			P("    Area = h*(a+b)/2");
+			P("    Area = h*(a + b)/2");
 			P("Circle of radius \"r\":");
 			P("    Area = pi*r^2                 Perimeter = 2*pi*r");
 			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
@@ -429,11 +436,11 @@ char	*cp;
 			P("Right circular cone of radius \"r\" and height \"h\":");
 			P("    Volume = pi*r^2*h/3");
 			P("    Base surface area = pi*r^2    Side surface area = pi*r*(r^2 + h^2)^.5\n");
-			P("Convex polygon of \"n\" sides sum of all interior angles formula:");
-			P("    Sum = (n-2)*180 degrees");
+			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
+			P("    Sum = (n - 2)*180 degrees     Sum = (n - 2)*pi radians");
 			return true;
 		}
-		if (strncasecmp(cp, "expressions", cp1 - cp) == 0) {
+		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 			P("    - negate");
@@ -444,8 +451,8 @@ char	*cp;
 			P("Operators in the same precedence level are evaluated left to right.\n");
 			P("Variables consist of any combination of letters, digits, underscores (_),");
 			P("and backslashes (\\).  Predefined constants and variables:\n");
-			P("    e# - the universal constant e (2.718281828...).");
-			P("    pi - the universal constant pi (3.1415926...).");
+			P("    e# - the universal constant e (2.7182818284...).");
+			P("    pi - the universal constant pi (3.1415926535...).");
 			P("    i# - imaginary number (square root of -1).");
 			P("    sign, sign1, sign2, sign3, ... - may be +1 or -1.");
 			P("    integer - may be any integer.");
@@ -455,7 +462,7 @@ char	*cp;
 		}
 		if (is_all(cp)) {
 			for (i = 0, row = 1;; i++, row += 3) {
-				if (gfp == stdout && screen_rows && row >= (screen_rows - 3)) {
+				if (gfp == stdout && screen_rows && row >= (screen_rows - 4)) {
 					row = 1;
 					if (!pause_cmd(""))
 						return false;
@@ -463,9 +470,7 @@ char	*cp;
 				}
 				if (i >= ARR_CNT(com_list))
 					break;
-				if (com_list[i].info) {
-					fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
-				}
+				fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
 				fprintf(gfp, "Usage: %s %s\n\n", com_list[i].name, com_list[i].usage);
 			}
 			P("End of command list.");
@@ -478,7 +483,7 @@ char	*cp;
 	P("Type \"help usage\" for syntax of all commands.");
 	P("Type \"help expressions\" for help with entering expressions.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
-	P("\"help\" followed by a command name will give info on a command.\n");
+	P("\"help\" or \"?\" followed by a command name will give info on a command.\n");
 	P("Available commands:");
 	for (i = 0; i < ARR_CNT(com_list); i++) {
 		if ((i % 5) == 0)
@@ -488,6 +493,6 @@ char	*cp;
 			fprintf(gfp, " ");
 	}
 	P("\n\nTo select an equation in memory, just enter the equation number.");
-	P("To solve an equation, simply type the variable name at the prompt.");
+	P("To solve the current equation, type the variable name at the prompt.");
 	return true;
 }
