@@ -1,9 +1,9 @@
 /*
- * Algebraic manipulator integration routines and commands.
+ * Mathomatic integration routines and commands.
  *
  * These are very low level, so they don't do much, polynomials only.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -19,7 +19,7 @@ static double	integrate_order;	/* global order of integration */
  * unless it is on the right side of a power operator.
  */
 make_powers(equation, np, v)
-token_type	*equation;	/* equation side pointer */
+token_type	*equation;	/* pointer to beginning of equation side */
 int		*np;		/* pointer to length of equation side */
 long		v;		/* variable */
 {
@@ -64,7 +64,7 @@ long		v;		/* variable */
  */
 int
 int_dispatch(equation, np, v, func)
-token_type	*equation;	/* equation side to integrate */
+token_type	*equation;	/* pointer to beginning of equation side to integrate */
 int		*np;		/* pointer to length of equation side */
 long		v;		/* integration variable */
 int		(*func)();	/* integration function to call for each term */
@@ -102,11 +102,11 @@ int		(*func)();	/* integration function to call for each term */
  */
 static int
 integrate_sub(equation, np, loc, eloc, v)
-token_type	*equation;	/* equation side pointer */
+token_type	*equation;	/* pointer to beginning of equation side */
 int		*np;		/* pointer to length of equation side */
 int		loc;		/* beginning location of term */
 int		eloc;		/* end location of term */
-long		v;		/* variable */
+long		v;		/* variable of integration */
 {
 	int		i, j, k;
 	int		len;
@@ -214,14 +214,13 @@ long		v;		/* variable */
 	}
 	blt(&equation[eloc+2], &equation[eloc], (*np - eloc) * sizeof(token_type));
 	*np += 2;
-	k = eloc;
-	equation[k].level = mlevel;
-	equation[k].kind = OPERATOR;
-	equation[k].token.operatr = TIMES;
-	k++;
-	equation[k].level = mlevel;
-	equation[k].kind = VARIABLE;
-	equation[k].token.variable = v;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = OPERATOR;
+	equation[eloc].token.operatr = TIMES;
+	eloc++;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = VARIABLE;
+	equation[eloc].token.variable = v;
 	return true;
 }
 
@@ -233,7 +232,7 @@ integrate_cmd(cp)
 char	*cp;
 {
 	int		i, j;
-	long		v;
+	long		v = 0;
 	double		d1;
 	token_type	*source, *dest;
 	int		n1, n2, *nps, *np;
@@ -243,7 +242,8 @@ char	*cp;
 	if (current_not_defined()) {
 		return false;
 	}
-	if (def_flag = (strcmp_tospace(cp, "definite") == 0)) {
+	def_flag = (strcmp_tospace(cp, "definite") == 0);
+	if (def_flag) {
 		cp = skip_param(cp);
 	}
 	i = next_espace();
@@ -283,7 +283,7 @@ char	*cp;
 	n1 = *nps;
 	for (d1 = 0; d1 < integrate_order; d1++) {
 		if (!int_dispatch(dest, &n1, v, integrate_sub)) {
-			error(_("Integration failed."));
+			error(_("Integration failed, not a polynomial."));
 			return false;
 		}
 		simp_loop(dest, &n1);
@@ -324,8 +324,7 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -396,7 +395,7 @@ long		v;
 			k++;
 			equation[k].level = mlevel + 1;
 			equation[k].kind = CONSTANT;
-			equation[k].token.constant = 0.0;
+			equation[k].token.constant = 1.0;
 			k++;
 			return true;
 		}
@@ -406,14 +405,13 @@ long		v;
 	}
 	blt(&equation[eloc+2], &equation[eloc], (*np - eloc) * sizeof(token_type));
 	*np += 2;
-	k = eloc;
-	equation[k].level = mlevel;
-	equation[k].kind = OPERATOR;
-	equation[k].token.operatr = DIVIDE;
-	k++;
-	equation[k].level = mlevel;
-	equation[k].kind = VARIABLE;
-	equation[k].token.variable = v;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = OPERATOR;
+	equation[eloc].token.operatr = DIVIDE;
+	eloc++;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = VARIABLE;
+	equation[eloc].token.variable = v;
 	return true;
 }
 
@@ -463,7 +461,6 @@ long		v;
 			equation[j].level = level;
 			equation[j].kind = CONSTANT;
 			equation[j].token.constant = 1.0;
-			j++;
 			blt(&equation[eloc+len+3], &equation[eloc], (*np - eloc) * sizeof(token_type));
 			*np += len + 3;
 			k = eloc;
@@ -479,8 +476,7 @@ long		v;
 			k++;
 			equation[k].level = mlevel + 1;
 			equation[k].kind = CONSTANT;
-			equation[k].token.constant = 0.0;
-			k++;
+			equation[k].token.constant = 1.0;
 			return true;
 		}
 	}
@@ -495,7 +491,7 @@ laplace_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v;
+	long		v = 0;
 	int		inverse_flag;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
@@ -539,15 +535,13 @@ char	*cp;
 	blt(dest, source, *nps * sizeof(token_type));
 	n1 = *nps;
 	if (inverse_flag) {
-		if (!poly_in_v(dest, n1, v, true)
-		    || !int_dispatch(dest, &n1, v, inv_laplace_sub)) {
+		if (!poly_in_v(dest, n1, v, true) || !int_dispatch(dest, &n1, v, inv_laplace_sub)) {
 			error(_("Inverse Laplace failed."));
 			return false;
 		}
 	} else {
-		if (!poly_in_v(dest, n1, v, false)
-		    || !int_dispatch(dest, &n1, v, laplace_sub)) {
-			error(_("Laplace failed."));
+		if (!poly_in_v(dest, n1, v, false) || !int_dispatch(dest, &n1, v, laplace_sub)) {
+			error(_("Laplace failed, not a polynomial."));
 			return false;
 		}
 	}
@@ -558,11 +552,10 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
-#if	!BASICS && !LIBRARY
+#if	!LIBRARY
 /*
  * Numerical integrate command.
  */
@@ -570,7 +563,7 @@ int
 nintegrate_cmd(cp)
 char	*cp;
 {
-	long		v;
+	long		v = 0;
 	int		i, j, k, i1, i2;
 	int		level;
 	int		iterations;
@@ -647,7 +640,7 @@ char	*cp;
 		error(_("Error: Bound contains infinity."));
 		return false;
 	}
-	if ((n_tlhs + 1 + n_trhs + 2) > n_tokens) {
+	if ((n_tlhs + n_trhs + 3) > n_tokens) {
 		error_huge();
 	}
 	if (trap_flag) {
@@ -758,11 +751,11 @@ char	*cp;
 			n1 += 2;
 		}
 
-		in_calc_cmd = true;
+		approximate_roots = true;
 		elim_loop(dest, &n1);
 		ufactor(dest, &n1);
 		simp_divide(dest, &n1);
-		in_calc_cmd = false;
+		approximate_roots = false;
 
 		if (exp_contains_infinity(dest, n1)) {
 			error(_("Integration failed because result contains infinity or nan."));
@@ -811,11 +804,11 @@ char	*cp;
 	for (; k < n1; k++)
 		dest[k].level++;
 
-	in_calc_cmd = true;
+	approximate_roots = true;
 	elim_loop(dest, &n1);
 	ufactor(dest, &n1);
 	simp_divide(dest, &n1);
-	in_calc_cmd = false;
+	approximate_roots = false;
 
 	debug_string(0, _("Integration successful."));
 	if (n_rhs[cur_equation]) {
@@ -824,7 +817,6 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 #endif
