@@ -1,8 +1,8 @@
 /*
- * Algebraic manipulator expression and equation display routines.
+ * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
@@ -39,6 +39,7 @@ reset_attr()
 			printf("\033[0m");
 		}
 	}
+	fflush(NULL);	/* flush all output */
 }
 
 /*
@@ -78,13 +79,14 @@ default_color()
 }
 
 /*
- * Display the expression or equation stored in equation space "n" in single line format.
+ * Display the expression or equation stored in equation space "n" in single-line format.
+ * The equation space is not modified.
  *
  * Return true if successful.
  */
 int
 list1_sub(n, export_flag)
-int	n;		/* equation number */
+int	n;		/* equation space number */
 int	export_flag;	/* true for exportable format (readable by other math programs) */
 			/* 1 for Maxima, 2 for other */
 {
@@ -120,21 +122,21 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
  */
 int
 list_sub(n)
-int	n;	/* equation number */
+int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
 		return false;
-	if (groupall) {
+	if (display2d) {
+		/* display in fraction format */
 		group_sub(n);
 		if (factor_int_flag) {
-			factor_int(lhs[n], &n_lhs[n]);
-			factor_int(rhs[n], &n_rhs[n]);
+			factor_int_sub(n);
 		}
 		return flist_sub(n);
 	} else {
+		/* display in single-line format */
 		if (factor_int_flag) {
-			factor_int(lhs[n], &n_lhs[n]);
-			factor_int(rhs[n], &n_rhs[n]);
+			factor_int_sub(n);
 		}
 		return list1_sub(n, false);
 	}
@@ -209,15 +211,15 @@ int	lang_code;	/* language code */
 		break;
 	case V_E:
 		switch (lang_code) {
-		case -2:
-			cp = "e";
-			break;
 		case -1:
 			cp = "%e";
 			break;
 		case 0:
 			cp = "e#";
 			break;
+		case 1:
+			cp ="M_E";
+			break;
 		case 2:
 			cp = "Math.E";
 			break;
@@ -225,7 +227,7 @@ int	lang_code;	/* language code */
 			cp = "math.e";
 			break;
 		default:
-			cp = "E";
+			cp = "e";
 			break;
 		}
 		break;
@@ -235,7 +237,7 @@ int	lang_code;	/* language code */
 			cp = "%pi";
 			break;
 		case 1:
-			cp = "PI";
+			cp = "M_PI";
 			break;
 		case 2:
 			cp = "Math.PI";
@@ -290,6 +292,7 @@ int	lang_code;	/* language code */
 /*
  * Display an expression in single line format.
  * Use color if available.
+ * The expression is not modified.
  *
  * Return number of characters output (excluding escape sequences).
  */
@@ -323,8 +326,11 @@ int		export_flag;	/* flag for exportable format (usually false) */
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
+			if (equation[i].token.constant == 0.0) {
+				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant <= 0.0) {
+				if (equation[i].token.constant < 0.0) {
 					len += fprintf(gfp, "(%.20g)", equation[i].token.constant);
 				} else {
 					len += fprintf(gfp, "%.20g", equation[i].token.constant);
@@ -332,7 +338,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 			} else if (finance_option) {
 				len += fprintf(gfp, "%.2f", equation[i].token.constant);
 			} else {
-				len += fprintf(gfp, "%.12g", equation[i].token.constant);
+				len += fprintf(gfp, "%.*g", precision, equation[i].token.constant);
 			}
 			break;
 		case VARIABLE:
@@ -358,11 +364,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				cp = " % ";
 				break;
 			case POWER:
-				if (starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
+				cp = "^";
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -389,7 +391,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
  */
 char *
 list_equation(n, export_flag)
-int	n;		/* equation number */
+int	n;		/* equation space number */
 int	export_flag;	/* flag for exportable format (usually false) */
 {
 	int	len;
@@ -402,8 +404,10 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	}
 	len += 2;
 	cp = (char *) malloc(len);
-	if (cp == NULL)
+	if (cp == NULL) {
+		error(_("Can't malloc()."));
 		return NULL;
+	}
 	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
 	if (n_rhs[n]) {
 		strcat(cp, " = ");
@@ -415,7 +419,6 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	return cp;
 }
 
-#if	false
 /*
  * Store an expression in a text string.
  * String should be freed with free() when done.
@@ -434,12 +437,13 @@ int		export_flag;
 	len = list_string_sub(equation, n, NULL, export_flag);
 	len++;
 	cp = (char *) malloc(len);
-	if (cp == NULL)
+	if (cp == NULL) {
+		error(_("Can't malloc()."));
 		return NULL;
+	}
 	list_string_sub(equation, n, cp, export_flag);
 	return cp;
 }
-#endif
 
 /*
  * Convert an expression to a text string and store in "string" if
@@ -483,8 +487,11 @@ int		export_flag;
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
+			if (equation[i].token.constant == 0.0) {
+				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (high_prec || export_flag) {
-				if (equation[i].token.constant <= 0.0) {
+				if (equation[i].token.constant < 0.0) {
 					len += snprintf(buf, sizeof(buf), "(%.20g)", equation[i].token.constant);
 				} else {
 					len += snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
@@ -492,7 +499,7 @@ int		export_flag;
 			} else if (finance_option) {
 				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.12g", equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
 			}
 			if (string) {
 				strcat(string, buf);
@@ -522,11 +529,7 @@ int		export_flag;
 				cp = " % ";
 				break;
 			case POWER:
-				if (starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
+				cp = "^";
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -557,14 +560,11 @@ token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 {
 	int	i;
-	double	d;
 
 	for (i = 0; i < n; i++) {
-		if ((equation[i].kind == CONSTANT
-		    && fmod(equation[i].token.constant, 1.0) != 0.0)
-		    || (equation[i].kind == VARIABLE
-		    && var_is_const(equation[i].token.variable, &d))) {
-			return false;	/* non-integer constant */
+		if ((equation[i].kind == CONSTANT && fmod(equation[i].token.constant, 1.0))
+		    || (equation[i].kind == VARIABLE && equation[i].token.variable <= IMAGINARY)) {
+			return false;
 		}
 	}
 	return true;
@@ -573,19 +573,22 @@ int		n;		/* length of expression */
 /*
  * Display an equation as C, Java, or Python code.
  */
-list_c_equation(en, java_flag, int_flag)
-int	en;		/* equation number */
-int	java_flag;
+list_c_equation(en, language, int_flag)
+int	en;		/* equation space number */
+int	language;
 int	int_flag;	/* integer arithmetic flag */
 {
 	if (n_lhs[en] <= 0)
 		return;
-	list_code(lhs[en], &n_lhs[en], java_flag, int_flag);
+	list_code(lhs[en], &n_lhs[en], language, int_flag);
 	if (n_rhs[en]) {
 		fprintf(gfp, " = ");
-		list_code(rhs[en], &n_rhs[en], java_flag, int_flag);
+		list_code(rhs[en], &n_rhs[en], language, int_flag);
 	}
-	fprintf(gfp, ";\n");
+	if (language != 2) {
+		fprintf(gfp, ";");
+	}
+	fprintf(gfp, "\n");
 }
 
 /*
@@ -595,10 +598,10 @@ int	int_flag;	/* integer arithmetic flag */
  * Return length of output (number of characters).
  */
 int
-list_code(equation, np, java_flag, int_flag)
+list_code(equation, np, language, int_flag)
 token_type	*equation;	/* expression pointer */
 int		*np;		/* pointer to length of expression */
-int		java_flag;	/* if 1, generate Java code, if 2, generate Python code */
+int		language;	/* if 0, generate C code; if 1, generate Java code; if 2, generate Python code */
 int		int_flag;	/* integer arithmetic flag, should work with any language */
 {
 	int	i, j, k, i1, i2;
@@ -633,9 +636,9 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 								equation[i2].token.operatr = TIMES;
 								equation[i2+1] = equation[i2-1];
 							} else {
-								if (int_flag || java_flag > 1)
+								if (int_flag || language > 1)
 									break;
-								if (java_flag) {
+								if (language) {
 									cp = "Math.pow";
 								} else {
 									cp = "pow";
@@ -657,10 +660,13 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
+			if (equation[i].token.constant == 0.0) {
+				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (int_flag) {
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
-				snprintf(buf, sizeof(buf), "%#.16g", equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%#.*g", MAX_PRECISION, equation[i].token.constant);
 				j = strlen(buf) - 1;
 				for (; j >= 0; j--) {
 					if (buf[j] == '0')
@@ -675,7 +681,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			len += fprintf(gfp, "%s", buf);
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, java_flag + 1);
+			len += list_var(equation[i].token.variable, language + 1);
 			fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
@@ -697,7 +703,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 				cp = " % ";
 				break;
 			case POWER:
-				if (int_flag || java_flag > 1) {
+				if (int_flag || language > 1) {
 					cp = " ** ";
 				} else {
 					cp = ", ";
@@ -729,7 +735,7 @@ int	cur_pos;	/* current position in the current line on the screen */
  */
 int
 flist_sub(n)
-int	n;	/* equation number */
+int	n;	/* equation space number */
 {
 	int	i;
 	int	sind;
@@ -960,12 +966,13 @@ check_again:
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
-			if (equation[i].token.constant == 0.0)
+			if (equation[i].token.constant == 0.0) {
 				equation[i].token.constant = 0.0; /* fix -0 */
+			}
 			if (finance_option) {
 				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.12g", equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
 			}
 			if (oflag)
 				fprintf(gfp, "%s", buf);
@@ -994,11 +1001,7 @@ check_again:
 				cp = " % ";
 				break;
 			case POWER:
-				if (starstar) {
-					cp = "**";
-				} else {
-					cp = "^";
-				}
+				cp = "^";
 				break;
 			case FACTORIAL:
 				cp = "!";
