@@ -1,8 +1,8 @@
 /*
- * This file contains main() and startup code for Mathomatic.
+ * This file contains main() and the startup code for Mathomatic.
  * Leave this file out when compiling for the symbolic math library.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  *
  * Output to stderr is only done in this file.  The rest of the code
  * should not output to stderr.
@@ -90,12 +90,13 @@ char	**argv;
 			coption = true;
 			break;
 		case 'h':
+			readline_enabled = false;
 			html_flag = true;
 			break;
 		case 'm':
 			multiplier = atof(optarg);
-			if (multiplier <= 0.0 || (n_tokens = (int) (multiplier * DEFAULT_N_TOKENS)) <= 0) {
-				fprintf(stderr, _("Invalid memory size multiplier specified!\n"));
+			if (multiplier <= 0.0 || (n_tokens = (int) (multiplier * DEFAULT_N_TOKENS)) < 100) {
+				fprintf(stderr, _("Invalid memory size multiplier specified.\n"));
 				usage();
 			}
 			break;
@@ -103,6 +104,7 @@ char	**argv;
 			quiet_mode = true;
 			break;
 		case 't':
+			readline_enabled = false;
 			test_mode = true;
 			break;
 		case 'u':
@@ -144,7 +146,7 @@ char	**argv;
 #endif
 		printf(_("Mathomatic version %s (www.mathomatic.org)\n"), VERSION);
 		printf(_("Copyright (C) 1987-2006 George Gesslein II.\n"));
-		printf(_("%d equation spaces available, %ldK per equation space.\n\n"),
+		printf(_("%d equation spaces available, %ldKB per equation space.\n\n"),
 		    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
 	}
 #if	!SECURE
@@ -173,7 +175,7 @@ char	**argv;
 		}
 	} else {
 		if (!set_signals()) {
-			fprintf(stderr, _("Signal(2) setting failed.\n"));
+			fprintf(stderr, _("signal(2) setting failed.\n"));
 		}
 		if (!f_to_fraction(0.5, &numerator, &denominator)
 		    || !f_to_fraction(1.0/3.0, &numerator, &denominator)) {
@@ -209,7 +211,7 @@ usage()
 	fprintf(stderr, _("Usage: %s [ options ] [ input_files ]\n\n"), prog_name);
 	fprintf(stderr, _("Options:\n"));
 	fprintf(stderr, _("  -c               Toggle color mode.\n"));
-	fprintf(stderr, _("  -h               Enable HTML mode.\n"));
+	fprintf(stderr, _("  -h               Enable HTML mode and disable readline input.\n"));
 	fprintf(stderr, _("  -m number        Specify a memory size multiplier.\n"));
 	fprintf(stderr, _("  -q               Quiet mode (don't display prompts).\n"));
 	fprintf(stderr, _("  -t               Used when testing.\n"));
@@ -271,7 +273,7 @@ load_rc()
 	if (fp == NULL)
 		return true;
 #endif
-	while (cp = fgets(buf, sizeof(buf), fp)) {
+	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
 		set_error_level(cp);
 		if (!set_options(cp))
 			rv = false;
