@@ -1,12 +1,24 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 #include "includes.h"
 
-int	point_flag;	/* point to error if true */
+#define	INFINITY_NAME	"inf"	/* set this to the name of the infinity constant as displayed by printf(3) */
+
+int	point_flag;		/* point to location of parse error if true */
+
+/*
+ * Return true if character is a valid starting variable character.
+ */
+static inline int
+isvarchar(ch)
+int	ch;
+{
+	return(ch == '_' || ch == '\\' || isalpha(ch));
+}
 
 /*
  * Parse an equation string into equation space "n".
@@ -23,6 +35,8 @@ char	*cp;	/* pointer to the beginning of the equation character string */
 	}
 	if ((cp = parse_section(lhs[n], &n_lhs[n], cp)) != NULL) {
 		if ((cp = parse_section(rhs[n], &n_rhs[n], cp)) != NULL) {
+			if (extra_garbage(cp))
+				return NULL;
 			return cp;
 		}
 	}
@@ -39,8 +53,8 @@ char	*cp;	/* pointer to the beginning of the equation character string */
  */
 char	*
 parse_section(equation, np, cp)
-token_type	*equation;	/* where the parsed expression is stored */
-int		*np;		/* pointer to parsed expression length */
+token_type	*equation;	/* where the parsed expression is stored (equation side) */
+int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
 {
 	int		n = 0;		/* position in equation[] */
@@ -129,13 +143,17 @@ char		*cp;		/* string to parse */
 			if (operand) {
 				goto syntax_error;
 			}
+			if (cp[1] == '!') {
+				put_up_arrow((int) (cp - cp_start), _("Multifactorial not supported yet."));
+				return(NULL);
+			}
 			equation[n].level = cur_level;
 			equation[n].kind = OPERATOR;
 			equation[n].token.operatr = FACTORIAL;
 			n++;
 			equation[n].level = cur_level;
 			equation[n].kind = CONSTANT;
-			equation[n].token.constant = 0.0;
+			equation[n].token.constant = 1.0;
 			n++;
 			operand = true;
 			break;
@@ -222,7 +240,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (*cp == '-' && !((isascii(cp[1]) && isdigit(cp[1])) || cp[1] == '.')) {
+			if (*cp == '-' && !(isdigit(cp[1]) || cp[1] == '.')) {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = -1.0;
 				equation[n].level = cur_level;
@@ -253,7 +271,7 @@ parse_power:
 			break;
 		default:
 			if (!isvarchar(*cp)) {
-				put_up_arrow((int) (cp - cp_start), _("Unsupported character encountered."));
+				put_up_arrow((int) (cp - cp_start), _("Unrecognized character."));
 				return(NULL);
 			}
 			if (!operand) {
@@ -263,16 +281,16 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (strncmp(cp, "inf", 3) == 0 && !isvarchar(cp[3])) {
+			if (strncmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
+			    && !isvarchar(cp[strlen(INFINITY_NAME)])) {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = HUGE_VAL;	/* the infinity constant */
-				cp += 3;
+				cp += strlen(INFINITY_NAME);
 			} else {
 				equation[n].kind = VARIABLE;
 				cp1 = cp;
 				cp = parse_var(&equation[n].token.variable, cp);
 				if (cp == NULL) {
-					put_up_arrow((int) (cp1 - cp_start), _("Invalid variable."));
 					return(NULL);
 				}
 			}
@@ -292,9 +310,10 @@ p_out:
 	}
 	if (*cp == '=')
 		cp++;
-	if (*np = n) {
-		handle_negate(equation, *np);
-		prior_sub(equation, *np);
+	*np = n;
+	if (n) {
+		handle_negate(equation, np);
+		prior_sub(equation, np);
 		organize(equation, np);
 	}
 	input_column += (cp - cp_start);
@@ -309,7 +328,8 @@ syntax_error:
  * Parse variable name pointed to by "cp".
  * Variable name is converted to Mathomatic format and stored in "*vp".
  *
- * Return new string position, or NULL on failure.
+ * Return new string position if successful.
+ * Display error message and return NULL on failure.
  */
 char	*
 parse_var(vp, cp)
@@ -328,10 +348,8 @@ char	*cp;
 	} else {
 		strcmpfunc = strcasecmp;
 	}
-	if (vp == NULL || cp == NULL) {
-		return(NULL);
-	}
 	if (!isvarchar(*cp)) {
+		error(_("Invalid variable."));
 		return(NULL);	/* variable name must start with a valid variable character */
 	}
 	cp1 = cp;
@@ -340,12 +358,16 @@ char	*cp;
 			break;
 		}
 		if (i >= MAX_VAR_LEN) {
+			error(_("Variable name too long."));
 			return(NULL);
 		}
 		buf[i++] = *cp1++;
 	}
 	buf[i] = '\0';
-	if ((*strcmpfunc)(buf, "sign") == 0) {
+	if ((*strcmpfunc)(buf, INFINITY_NAME) == 0) {
+		error(_("Invalid variable."));
+		return(NULL);
+	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
 		cp += strlen(buf);
 	} else if ((*strcmpfunc)(buf, "integer") == 0) {
@@ -371,10 +393,11 @@ char	*cp;
 			return(cp + 2);
 		}
 		for (i = 0; *cp;) {
-			if (!isvarchar(*cp) && !(isascii(*cp) && isdigit(*cp))) {
+			if (!isvarchar(*cp) && !isdigit(*cp)) {
 				break;
 			}
 			if (i >= MAX_VAR_LEN) {
+				error(_("Variable name too long."));
 				return(NULL);
 			}
 			buf[i++] = *cp++;
@@ -398,7 +421,7 @@ char	*cp;
 			len = strlen(buf) + 1;
 			var_names[i] = (char *) malloc(len);
 			if (var_names[i] == NULL) {
-				error(_("Out of memory!  (can't malloc())."));
+				error(_("Out of memory (can't malloc(3) variable name)."));
 				return(NULL);
 			}
 			blt(var_names[i], buf, len);
@@ -406,9 +429,10 @@ char	*cp;
 			var_names[i+1] = NULL;
 		}
 	}
-	if (isascii(*cp) && isdigit(*cp)) {
+	if (isdigit(*cp)) {
 		j = strtol(cp, &cp, 10);
 		if (j < 0 || j > MAX_SUBSCRIPT) {
+			error(_("Maximum subscript exceeded in special variable name."));
 			return(NULL);
 		}
 		if (vtmp == SIGN) {
@@ -422,6 +446,7 @@ char	*cp;
 
 /*
  * Set point_flag if pointing to the input error works for the passed string.
+ * Truncate string to the actual content.
  */
 set_error_level(cp)
 char	*cp;	/* input string */
@@ -429,21 +454,25 @@ char	*cp;	/* input string */
 	char	*cp1;
 
 	point_flag = true;
-/* remove trailing newlines */
-	cp1 = &cp[strlen(cp)];
-	while (cp1 > cp) {
-		cp1--;
-		if (*cp1 == '\n' || *cp1 == '\r') {
-			*cp1 = '\0';
-		} else
-			break;
-	}
 /* handle comments */
 	for (cp1 = cp; *cp1; cp1++) {
 		if (*cp1 == ';') {
 			*cp1 = '\0';
 			break;
 		}
+	}
+/* remove trailing newlines, carriage returns, and spaces */
+	cp1 = &cp[strlen(cp)];
+	while (cp1 > cp) {
+		cp1--;
+		if (*cp1 == '\n' || *cp1 == '\r' || isspace(*cp1)) {
+			*cp1 = '\0';
+		} else {
+			break;
+		}
+	}
+/* set point_flag to false if non-printable characters encountered */
+	for (cp1 = cp; *cp1; cp1++) {
 		if (!isprint(*cp1)) {
 			point_flag = false;
 		}
@@ -481,10 +510,10 @@ double	*dp;
 {
 	switch (v) {
 	case V_E:
-		*dp = E;
+		*dp = M_E;
 		return true;
 	case V_PI:
-		*dp = PI;
+		*dp = M_PI;
 		return true;
 	}
 	return false;
@@ -545,16 +574,16 @@ int		i;		/* location of operator to parenthesize in expression */
 /*
  * Handle and remove the special NEGATE operator.
  */
-handle_negate(equation, n)
+handle_negate(equation, np)
 token_type	*equation;
-int		n;
+int		*np;
 {
 	int	i;
 
-	for (i = 1; i < n; i += 2) {
+	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == NEGATE) {
 			equation[i].token.operatr = TIMES;
-			binary_parenthesize(equation, n, i);
+			binary_parenthesize(equation, *np, i);
 		}
 	}
 }
@@ -565,28 +594,28 @@ int		n;
  *
  * organize() should be called after this.
  */
-prior_sub(equation, n)
+prior_sub(equation, np)
 token_type	*equation;
-int		n;
+int		*np;
 {
 	int	i;
 
-	for (i = 1; i < n; i += 2) {
+	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == FACTORIAL) {
-			binary_parenthesize(equation, n, i);
+			binary_parenthesize(equation, *np, i);
 		}
 	}
-	for (i = 1; i < n; i += 2) {
+	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == POWER) {
-			binary_parenthesize(equation, n, i);
+			binary_parenthesize(equation, *np, i);
 		}
 	}
-	for (i = 1; i < n; i += 2) {
+	for (i = 1; i < *np; i += 2) {
 		switch (equation[i].token.operatr) {
 		case TIMES:
 		case DIVIDE:
 		case MODULUS:
-			binary_parenthesize(equation, n, i);
+			binary_parenthesize(equation, *np, i);
 			break;
 		}
 	}
@@ -599,25 +628,11 @@ str_tolower(cp)
 char	*cp;
 {
 	for (; *cp; cp++) {
-		if (isascii(*cp) && isupper(*cp))
+		if (isupper(*cp))
 			*cp = tolower(*cp);
 	}
 }
 
-/*
- * Return true if character is a valid starting variable character.
- */
-int
-isvarchar(ch)
-int	ch;
-{
-	if (ch == '_' || ch == '\\')
-		return true;
-	if (isascii(ch) && isalpha(ch))
-		return true;
-	return false;
-}
-
 #ifndef	my_strlcpy
 /*
  * A very efficient strlcpy().
