@@ -1,6 +1,6 @@
 /*
- * Generate consecutive prime numbers using a modified sieve algorithm
- * that doesn't use much memory by using a windowing sieve buffer.
+ * Generate consecutive prime numbers using a modified sieve of Eratosthenes
+ * algorithm that doesn't use much memory by using a windowing sieve buffer.
  * Works with up to 19 digit numbers.
  *
  * Copyright (C) 2006 George Gesslein II.
@@ -57,7 +57,7 @@
 #define	MAX_K_INTEGER	1.0e19L		/* 19 digits for long doubles */
 
 /* memory usage in bytes; can be set to any size; the larger, the faster */
-#define BUFFER_SIZE	5000000
+#define BUFFER_SIZE	2000000
 #if	BUFFER_SIZE >= (INT_MAX / 2)
 #error	BUFFER_SIZE too big!
 #endif
@@ -65,12 +65,13 @@
 void	generate_primes(void);
 int	test_pal(long double d, long double base);
 void	usage(void);
+void	usage2(void);
 int	get_long_double_int(char *cp, long double *dp);
 
 long double strtold(const char *nptr, char **endptr);
 
 long double start_value, number, end_value;
-long double sq[] = {			/* Additive array that skips over multiples of 2, 3, 5, and 7. */
+long double sq[] = {		/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
 	 2, 6, 4, 6, 8, 4, 2, 4,
@@ -114,11 +115,11 @@ main(int argc, char *argv[])
 		case 'p':
 			pal_flag = true;
 			if (!get_long_double_int(optarg, &pal_base)) {
-				usage();
+				usage2();
 			}
 			break;
 		default:
-			usage();
+			usage2();
 		}
 	}
 	if (argc > optind) {
@@ -169,8 +170,8 @@ main(int argc, char *argv[])
 	if (argc > optind) {
 		usage();
 	}
-	if (pal_base < 2 || pal_base > 256) {
-		fprintf(stderr, "Palindromic base range is 2 to 256.\n");
+	if (pal_base < 2 || pal_base >= INT_MAX) {
+		fprintf(stderr, "Palindromic base must be >= 2.\n");
 		usage();
 	}
 	if (start_value < 0.0) {
@@ -207,7 +208,7 @@ get2:
 			}
 		}
 	}
-	buffer_size = min(BUFFER_SIZE, (end_value - start_value) + 1.0L);
+	buffer_size = (int) min(BUFFER_SIZE, (end_value - start_value) + 1.0L);
 	prime = (char *) malloc(buffer_size);
 	if (prime == NULL) {
 		fprintf(stderr, "%s: Not enough memory.\n", prog_name);
@@ -351,10 +352,6 @@ test_pal(long double d, long double base)
 		digits[i] = (int) fmodl(d, base);
 		d /= base;
 	}
-	if (i <= 1) {
-		/* less than 2 digits */
-		return false;
-	}
 	/* compare the array of digits[] end to end */
 	for (j = 0, i--; j < i; j++, i--) {
 		if (digits[i] != digits[j])
@@ -368,8 +365,19 @@ usage()
 {
 	fprintf(stderr, "\nUsage: %s [start [stop]] [\"twin\"] [\"pal\" [base]]\n\n", prog_name);
 	fprintf(stderr, "Generate consecutive prime numbers up to 19 digits.\n");
-	fprintf(stderr, "If \"twin\" is specified, display only twin primes.\n");
-	fprintf(stderr, "If \"pal\" is specified, display only palindromic primes.\n");
+	fprintf(stderr, "If \"twin\" is specified, output only twin primes.\n");
+	fprintf(stderr, "If \"pal\" is specified, output only palindromic primes.\n");
 	fprintf(stderr, "The palindromic base may be specified, the default is base 10.\n");
 	exit(1);
 }
+
+void
+usage2()
+{
+	fprintf(stderr, "\nUsage: %s [options] [start [stop]]\n\n", prog_name);
+	fprintf(stderr, "Generate consecutive prime numbers up to 19 digits.\n");
+	fprintf(stderr, "Options:\n");
+	fprintf(stderr, "  -t               Output only twin primes.\n");
+	fprintf(stderr, "  -p base          Output only palindromic primes.\n");
+	exit(1);
+}
