@@ -1,15 +1,15 @@
 /*
  * Global function prototypes for Mathomatic.
  *
- * Copyright (c) 1987-2005 George Gesslein II.
+ * Copyright (C) 1987-2006 George Gesslein II.
  */
 
 /* command function list */
 int		clear_cmd(), quit_cmd(), list_cmd(), simplify_cmd(), help_cmd(), eliminate_cmd();
-int		unfactor_cmd(), compare_cmd(), extrema_cmd();
-int		read_cmd(), flist_cmd(), calculate_cmd();
+int		fraction_cmd(), unfactor_cmd(), compare_cmd(), extrema_cmd();
+int		read_cmd(), flist_cmd(), calculate_cmd(), solve_cmd();
 int		factor_cmd(), derivative_cmd(), replace_cmd();
-int		save_cmd(), print_cmd(), taylor_cmd(), limit_cmd();
+int		save_cmd(), taylor_cmd(), limit_cmd();
 int		copy_cmd(), divide_cmd(), pause_cmd(), version_cmd();
 int		edit_cmd(), real_cmd(), imaginary_cmd(), tally_cmd();
 int		roots_cmd(), set_cmd(), code_cmd(), optimize_cmd(), push_cmd();
@@ -18,7 +18,7 @@ int		sum_cmd(), product_cmd(), integrate_cmd(), nintegrate_cmd(), laplace_cmd();
 /* various functions that don't return int */
 char		*skip_space(), *skip_param();
 char		*get_string();
-char		*parse_equation(), *parse_section(), *parse_var(), *parse_var2();
+char		*parse_equation(), *parse_section(), *parse_var(), *parse_var2(), *parse_expr();
 char		*list_expression(), *list_equation();
 double		gcd(), my_round(), multiply_out_unique();
 long		decstrtol();
